@@ -16,15 +16,13 @@
  */
 
 // written by Victor Julien
-extern crate libc;
 
-use nom::IResult;
-use log::*;
+use crate::log::*;
 
-use smb::smb::*;
-use smb::smb2::*;
-use smb::dcerpc_records::*;
-use smb::events::*;
+use crate::smb::smb::*;
+use crate::smb::smb2::*;
+use crate::smb::dcerpc_records::*;
+use crate::smb::events::*;
 
 pub const DCERPC_TYPE_REQUEST:              u8 = 0;
 pub const DCERPC_TYPE_PING:                 u8 = 1;
@@ -196,7 +194,7 @@ impl SMBTransactionDCERPC {
 
 impl SMBState {
     fn new_dcerpc_tx(&mut self, hdr: SMBCommonHdr, vercmd: SMBVerCmdStat, cmd: u8, call_id: u32)
-        -> (&mut SMBTransaction)
+        -> &mut SMBTransaction
     {
         let mut tx = self.new_tx();
         tx.hdr = hdr;
@@ -211,7 +209,7 @@ impl SMBState {
     }
 
     fn new_dcerpc_tx_for_response(&mut self, hdr: SMBCommonHdr, vercmd: SMBVerCmdStat, call_id: u32)
-        -> (&mut SMBTransaction)
+        -> &mut SMBTransaction
     {
         let mut tx = self.new_tx();
         tx.hdr = hdr;
@@ -259,7 +257,7 @@ pub fn smb_write_dcerpc_record<'b>(state: &mut SMBState,
 
     SCLogDebug!("called for {} bytes of data", data.len());
     match parse_dcerpc_record(data) {
-        IResult::Done(_, dcer) => {
+        Ok((_, dcer)) => {
             SCLogDebug!("DCERPC: version {}.{} write data {} => {:?}",
                     dcer.version_major, dcer.version_minor, dcer.data.len(), dcer);
 
@@ -268,7 +266,7 @@ pub fn smb_write_dcerpc_record<'b>(state: &mut SMBState,
             if dcer.packet_type == DCERPC_TYPE_REQUEST && dcer.first_frag == false {
                 SCLogDebug!("NOT the first frag. Need to find an existing TX");
                 match parse_dcerpc_request_record(dcer.data, dcer.frag_len, dcer.little_endian) {
-                    IResult::Done(_, recr) => {
+                    Ok((_, recr)) => {
                         let found = match state.get_dcerpc_tx(&hdr, &vercmd, dcer.call_id) {
                             Some(tx) => {
                                 SCLogDebug!("previous CMD {} found at tx {} => {:?}",
@@ -305,7 +303,7 @@ pub fn smb_write_dcerpc_record<'b>(state: &mut SMBState,
             match dcer.packet_type {
                 DCERPC_TYPE_REQUEST => {
                     match parse_dcerpc_request_record(dcer.data, dcer.frag_len, dcer.little_endian) {
-                        IResult::Done(_, recr) => {
+                        Ok((_, recr)) => {
                             SCLogDebug!("DCERPC: REQUEST {:?}", recr);
                             if let Some(SMBTransactionTypeData::DCERPC(ref mut tdn)) = tx.type_data {
                                 SCLogDebug!("first frag size {}", recr.data.len());
@@ -323,6 +321,7 @@ pub fn smb_write_dcerpc_record<'b>(state: &mut SMBState,
                         },
                         _ => {
                             tx.set_event(SMBEvent::MalformedData);
+                            tx.request_done = true;
                         },
                     }
                 },
@@ -333,7 +332,7 @@ pub fn smb_write_dcerpc_record<'b>(state: &mut SMBState,
                         parse_dcerpc_bind_record_big(dcer.data)
                     };
                     match brec {
-                        IResult::Done(_, bindr) => {
+                        Ok((_, bindr)) => {
                             SCLogDebug!("SMB DCERPC {:?} BIND {:?}", dcer, bindr);
 
                             if bindr.ifaces.len() > 0 {
@@ -355,17 +354,21 @@ pub fn smb_write_dcerpc_record<'b>(state: &mut SMBState,
                                 }
                                 bind_ifaces = Some(ifaces);
                             }
-                            tx.request_done = true;
                         },
                         _ => {
                             tx.set_event(SMBEvent::MalformedData);
                         },
                     }
+                    tx.request_done = true;
                 }
-                21...255 => {
+                21..=255 => {
                     tx.set_event(SMBEvent::MalformedData);
+                    tx.request_done = true;
+                },
+                _ => {
+                    // valid type w/o special processing
+                    tx.request_done = true;
                 },
-                _ => { }, // valid type w/o special processing
             }
         },
         _ => {
@@ -387,7 +390,7 @@ fn smb_dcerpc_response_bindack(
         ntstatus: u32)
 {
     match parse_dcerpc_bindack_record(dcer.data) {
-        IResult::Done(_, bindackr) => {
+        Ok((_, bindackr)) => {
             SCLogDebug!("SMB READ BINDACK {:?}", bindackr);
 
             let found = match state.get_dcerpc_tx(&hdr, &vercmd, dcer.call_id) {
@@ -461,7 +464,7 @@ fn dcerpc_response_handle<'b>(tx: &mut SMBTransaction,
     match dcer.packet_type {
         DCERPC_TYPE_RESPONSE => {
             match parse_dcerpc_response_record(dcer.data, dcer.frag_len) {
-                IResult::Done(_, respr) => {
+                Ok((_, respr)) => {
                     SCLogDebug!("SMBv1 READ RESPONSE {:?}", respr);
                     if let Some(SMBTransactionTypeData::DCERPC(ref mut tdn)) = tx.type_data {
                         SCLogDebug!("CMD 11 found at tx {}", tx.id);
@@ -480,7 +483,7 @@ fn dcerpc_response_handle<'b>(tx: &mut SMBTransaction,
         DCERPC_TYPE_BINDACK => {
             // handled elsewhere
         },
-        21...255 => {
+        21..=255 => {
             if let Some(SMBTransactionTypeData::DCERPC(ref mut tdn)) = tx.type_data {
                 tdn.set_result(dcer.packet_type);
             }
@@ -533,7 +536,7 @@ pub fn smb_read_dcerpc_record<'b>(state: &mut SMBState,
 
     } else {
         match parse_dcerpc_record(&data) {
-            IResult::Done(_, dcer) => {
+            Ok((_, dcer)) => {
                 SCLogDebug!("DCERPC: version {}.{} read data {} => {:?}",
                         dcer.version_major, dcer.version_minor, dcer.data.len(), dcer);
 
@@ -581,17 +584,14 @@ pub fn smb_read_dcerpc_record<'b>(state: &mut SMBState,
 /// Try to find out if the input data looks like DCERPC
 pub fn smb_dcerpc_probe<'b>(data: &[u8]) -> bool
 {
-    match parse_dcerpc_record(data) {
-        IResult::Done(_, recr) => {
-            SCLogDebug!("SMB: could be DCERPC {:?}", recr);
-            if recr.version_major == 5 && recr.version_minor < 3 &&
-               recr.frag_len > 0 && recr.packet_type <= 20
+    if let Ok((_, recr)) = parse_dcerpc_record(data) {
+        SCLogDebug!("SMB: could be DCERPC {:?}", recr);
+        if recr.version_major == 5 && recr.version_minor < 3 &&
+            recr.frag_len > 0 && recr.packet_type <= 20
             {
                 SCLogDebug!("SMB: looks like we have dcerpc");
                 return true;
             }
-        },
-        _ => { },
     }
     return false;
 }
