@@ -34,19 +34,9 @@
 
 #include "app-layer-ikev2.h"
 
-#ifdef HAVE_RUST
-
 #include "rust-ikev2-ikev2-gen.h"
 
 void RegisterIKEV2Parsers(void)
 {
     rs_register_ikev2_parser();
 }
-
-#else /* HAVE_RUST */
-
-void RegisterIKEV2Parsers(void)
-{
-}
-
-#endif /* HAVE_RUST */
