@@ -26,11 +26,9 @@
 
 void JsonDnsLogRegister(void);
 
-#ifdef HAVE_LIBJANSSON
 #include "app-layer-dns-common.h"
 
 json_t *JsonDNSLogQuery(void *txptr, uint64_t tx_id) __attribute__((nonnull));
 json_t *JsonDNSLogAnswer(void *txptr, uint64_t tx_id) __attribute__((nonnull));
-#endif
 
 #endif /* __OUTPUT_JSON_DNS_H__ */
