@@ -56,6 +56,7 @@
 #include "counters.h"
 
 int debuglog_enabled = 0;
+int threading_set_cpu_affinity = FALSE;
 
 /* Runmode Global Thread Names */
 const char *thread_name_autofp = "RX";
@@ -75,7 +76,7 @@ const char *thread_name_counter_wakeup = "CW";
  */
 typedef struct RunMode_ {
     /* the runmode type */
-    int runmode;
+    enum RunModes runmode;
     const char *name;
     const char *description;
     /* runmode function */
@@ -83,7 +84,7 @@ typedef struct RunMode_ {
 } RunMode;
 
 typedef struct RunModes_ {
-    int no_of_runmodes;
+    int cnt;
     RunMode *runmodes;
 } RunModes;
 
@@ -136,8 +137,6 @@ static const char *RunModeTranslateModeToName(int runmode)
             return "NAPATECH";
         case RUNMODE_UNITTEST:
             return "UNITTEST";
-        case RUNMODE_TILERA_MPIPE:
-            return "MPIPE";
         case RUNMODE_AFP_DEV:
             return "AF_PACKET_DEV";
         case RUNMODE_NETMAP:
@@ -155,8 +154,7 @@ static const char *RunModeTranslateModeToName(int runmode)
             return "WINDIVERT(DISABLED)";
 #endif
         default:
-            SCLogError(SC_ERR_UNKNOWN_RUN_MODE, "Unknown runtime mode. Aborting");
-            exit(EXIT_FAILURE);
+            FatalError(SC_ERR_UNKNOWN_RUN_MODE, "Unknown runtime mode. Aborting");
     }
 }
 
@@ -168,15 +166,14 @@ static const char *RunModeTranslateModeToName(int runmode)
  * \param runmode            The runmode type.
  * \param runmode_customd_id The runmode custom id.
  */
-static RunMode *RunModeGetCustomMode(int runmode, const char *custom_mode)
+static RunMode *RunModeGetCustomMode(enum RunModes runmode, const char *custom_mode)
 {
-    int i;
-
-    for (i = 0; i < runmodes[runmode].no_of_runmodes; i++) {
-        if (strcmp(runmodes[runmode].runmodes[i].name, custom_mode) == 0)
-            return &runmodes[runmode].runmodes[i];
+    if (runmode < RUNMODE_USER_MAX) {
+        for (int i = 0; i < runmodes[runmode].cnt; i++) {
+            if (strcmp(runmodes[runmode].runmodes[i].name, custom_mode) == 0)
+                return &runmodes[runmode].runmodes[i];
+        }
     }
-
     return NULL;
 }
 
@@ -225,7 +222,6 @@ void RunModeRegisterRunModes(void)
     RunModeIdsAFPRegister();
     RunModeIdsNetmapRegister();
     RunModeIdsNflogRegister();
-    RunModeTileMpipeRegister();
     RunModeUnixSocketRegister();
     RunModeIpsWinDivertRegister();
 #ifdef UNITTESTS
@@ -250,7 +246,7 @@ void RunModeListRunmodes(void)
     int j = 0;
     for ( ; i < RUNMODE_USER_MAX; i++) {
         int mode_displayed = 0;
-        for (j = 0; j < runmodes[i].no_of_runmodes; j++) {
+        for (j = 0; j < runmodes[i].cnt; j++) {
             if (mode_displayed == 1) {
                 printf("|                   ----------------------------------------------"
                        "-----------------------\n");
@@ -272,7 +268,7 @@ void RunModeListRunmodes(void)
         if (mode_displayed == 1) {
             printf("|-----------------------------------------------------------------"
                    "-----------------------\n");
-        } 
+        }
     }
 
     return;
@@ -318,9 +314,6 @@ void RunModeDispatch(int runmode, const char *custom_mode)
             case RUNMODE_DAG:
                 custom_mode = RunModeErfDagGetDefaultMode();
                 break;
-            case RUNMODE_TILERA_MPIPE:
-                custom_mode = RunModeTileMpipeGetDefaultMode();
-                break;
             case RUNMODE_NAPATECH:
                 custom_mode = RunModeNapatechGetDefaultMode();
                 break;
@@ -424,19 +417,18 @@ int RunModeNeedsBypassManager(void)
  * \param description Description for this runmode.
  * \param RunModeFunc The function to be run for this runmode.
  */
-void RunModeRegisterNewRunMode(int runmode, const char *name,
+void RunModeRegisterNewRunMode(enum RunModes runmode,
+                               const char *name,
                                const char *description,
                                int (*RunModeFunc)(void))
 {
-    void *ptmp;
     if (RunModeGetCustomMode(runmode, name) != NULL) {
-        SCLogError(SC_ERR_RUNMODE, "A runmode by this custom name has already "
-                   "been registered.  Please use an unique name");
-        return;
+        FatalError(SC_ERR_RUNMODE, "runmode '%s' has already "
+                   "been registered. Please use an unique name.", name);
     }
 
-    ptmp = SCRealloc(runmodes[runmode].runmodes,
-                     (runmodes[runmode].no_of_runmodes + 1) * sizeof(RunMode));
+    void *ptmp = SCRealloc(runmodes[runmode].runmodes,
+                     (runmodes[runmode].cnt + 1) * sizeof(RunMode));
     if (ptmp == NULL) {
         SCFree(runmodes[runmode].runmodes);
         runmodes[runmode].runmodes = NULL;
@@ -444,19 +436,18 @@ void RunModeRegisterNewRunMode(int runmode, const char *name,
     }
     runmodes[runmode].runmodes = ptmp;
 
-    RunMode *mode = &runmodes[runmode].runmodes[runmodes[runmode].no_of_runmodes];
-    runmodes[runmode].no_of_runmodes++;
+    RunMode *mode = &runmodes[runmode].runmodes[runmodes[runmode].cnt];
+    runmodes[runmode].cnt++;
+    memset(mode, 0x00, sizeof(*mode));
 
     mode->runmode = runmode;
     mode->name = SCStrdup(name);
     if (unlikely(mode->name == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate string");
-        exit(EXIT_FAILURE);
+        FatalError(SC_ERR_MEM_ALLOC, "Failed to allocate string");
     }
     mode->description = SCStrdup(description);
     if (unlikely(mode->description == NULL)) {
-        SCLogError(SC_ERR_MEM_ALLOC, "Failed to allocate string");
-        exit(EXIT_FAILURE);
+        FatalError(SC_ERR_MEM_ALLOC, "Failed to allocate string");
     }
     mode->RunModeFunc = RunModeFunc;
 
@@ -498,7 +489,20 @@ int RunModeOutputFiledataEnabled(void)
     return filedata_logger_count > 0;
 }
 
-bool IsRunModeOffline(int run_mode_to_check)
+bool IsRunModeSystem(enum RunModes run_mode_to_check)
+{
+    switch (run_mode_to_check) {
+        case RUNMODE_PCAP_FILE:
+        case RUNMODE_ERF_FILE:
+        case RUNMODE_ENGINE_ANALYSIS:
+            return false;
+            break;
+        default:
+            return true;
+    }
+}
+
+bool IsRunModeOffline(enum RunModes run_mode_to_check)
 {
     switch(run_mode_to_check) {
         case RUNMODE_CONF_TEST:
@@ -581,7 +585,10 @@ static void SetupOutput(const char *name, OutputModule *module, OutputCtx *outpu
                 module->ts_log_progress, module->TxLogCondition,
                 module->ThreadInit, module->ThreadDeinit,
                 module->ThreadExitPrintStats);
-        logger_bits[module->alproto] |= (1<<module->logger_id);
+        /* Not used with wild card loggers */
+        if (module->alproto != ALPROTO_UNKNOWN) {
+            logger_bits[module->alproto] |= (1<<module->logger_id);
+        }
     } else if (module->FiledataLogFunc) {
         SCLogDebug("%s is a filedata logger", module->name);
         OutputRegisterFiledataLogger(module->logger_id, module->name,
@@ -663,22 +670,6 @@ static void RunModeInitializeEveOutput(ConfNode *conf, OutputCtx *parent_ctx)
         /* Error is no registered loggers with this name
          * were found .*/
         if (!sub_count) {
-#ifndef HAVE_RUST
-            const char *rust_types[] = { "eve-log.smb", "eve-log.nfs",
-                "eve-log.dhcp", "eve-log.krb5", "eve-log.ikev2",
-                "eve-log.tftp", NULL, };
-            const char **iter = rust_types;
-            bool is_rust = false;
-            while (*iter) {
-                is_rust |= (strcmp(*iter, subname) == 0);
-                iter++;
-            }
-            if (is_rust) {
-                SCLogWarning(SC_WARN_RUST_NOT_AVAILABLE, "output "
-                    "module '%s' depends on Rust support", subname);
-                continue;
-            }
-#endif
             FatalErrorOnInit(SC_ERR_INVALID_ARGUMENT,
                     "No output module named %s", subname);
             continue;
@@ -754,7 +745,14 @@ void RunModeInitializeOutputs(void)
             continue;
         }
 
-        if (strncmp(output->val, "unified-", sizeof("unified-") - 1) == 0) {
+        if (strcmp(output->val, "file-log") == 0) {
+            SCLogWarning(SC_ERR_NOT_SUPPORTED,
+                    "file-log is no longer supported,"
+                    " use eve.files instead "
+                    "(see https://redmine.openinfosecfoundation.org/issues/2376"
+                    " for an explanation)");
+            continue;
+        } else if (strncmp(output->val, "unified-", sizeof("unified-") - 1) == 0) {
             SCLogWarning(SC_ERR_NOT_SUPPORTED,
                     "Unified1 is no longer supported,"
                     " use Unified2 instead "
@@ -768,14 +766,6 @@ void RunModeInitializeOutputs(void)
                     "recompile with --enable-prelude to add Prelude "
                     "support.");
             continue;
-#endif
-        } else if (strcmp(output->val, "eve-log") == 0) {
-#ifndef HAVE_LIBJANSSON
-            SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "Eve-log support not compiled in. Reconfigure/"
-                    "recompile with libjansson and its development "
-                    "files installed to add eve-log support.");
-            continue;
 #endif
         } else if (strcmp(output->val, "lua") == 0) {
 #ifndef HAVE_LUA
@@ -786,11 +776,9 @@ void RunModeInitializeOutputs(void)
             continue;
 #endif
         } else if (strcmp(output->val, "dns-log") == 0) {
-#ifdef HAVE_RUST
             SCLogWarning(SC_ERR_NOT_SUPPORTED,
-                    "dns-log is not available when Rust is enabled.");
+                    "dns-log is not longer available as of Suricata 5.0");
             continue;
-#endif
         } else if (strcmp(output->val, "tls-log") == 0) {
             tls_log_enabled = 1;
         }
@@ -809,7 +797,7 @@ void RunModeInitializeOutputs(void)
                 OutputInitResult r = module->InitFunc(output_config);
                 if (!r.ok) {
                     FatalErrorOnInit(SC_ERR_INVALID_ARGUMENT,
-                        "output module setup failed");
+                        "output module \"%s\": setup failed", output->val);
                     continue;
                 } else if (r.ctx == NULL) {
                     continue;
